@@ -36,8 +36,13 @@ import numbers
 import sys
 import time
 
-from talisker.context import ContextStack
-from talisker.util import module_dict, module_cache, get_errno_fields
+from talisker.context import ContextStack, track_request_metric
+from talisker.util import (
+    get_errno_fields,
+    get_rounded_ms,
+    module_cache,
+    module_dict,
+)
 
 
 __all__ = [
@@ -113,23 +118,28 @@ def configure(config):  # pragma: no cover
 
     set_logger_class()
     formatter = StructuredFormatter()
-    if config['color']:
-        formatter = ColoredFormatter(style=config['color'])
+    if config.colour:
+        formatter = ColouredFormatter(style=config.colour)
 
     # always INFO to stderr
     add_talisker_handler(logging.INFO, get_talisker_handler(), formatter)
 
-    configure_warnings(config['devel'])
+    configure_warnings(config.devel)
     supress_noisy_logs()
 
     # defer this until logging has been set up
     logger = logging.getLogger(__name__)
+    config_extra = {k: v.value for k, v in config.metadata().items() if v.raw}
+    if config_extra:
+        logger.info('talisker configured', extra=config_extra)
+    if config.ERRORS:
+        errors = {name: str(err) for name, err in config.ERRORS.items()}
+        logger.error('configuration errors', extra=errors)
 
-    debug = config['debuglog']
-    if debug is not None:
-        if can_write_to_file(debug):
+    if config.debuglog is not None:
+        if can_write_to_file(config.debuglog):
             handler = logging.handlers.TimedRotatingFileHandler(
-                debug,
+                config.debuglog,
                 when='D',
                 interval=1,
                 backupCount=1,
@@ -138,15 +148,16 @@ def configure(config):  # pragma: no cover
             )
             handler._debug_handler = True
             add_talisker_handler(logging.DEBUG, handler)
-            logger.info('enabling debug log', extra={'path': debug})
+            logger.info('enabling debug log', extra={'path': config.debuglog})
         else:
             logger.info('could not enable debug log, could not write to path',
-                        extra={'path': debug})
+                        extra={'path': config.debuglog})
 
     # sentry integration
     import talisker.sentry  # defer to avoid logging setup
-    sentry_handler = talisker.sentry.get_log_handler()
-    add_talisker_handler(logging.ERROR, sentry_handler)
+    if talisker.sentry.enabled:
+        sentry_handler = talisker.sentry.get_log_handler()
+        add_talisker_handler(logging.ERROR, sentry_handler)
 
     logging_globals['configured'] = True
 
@@ -176,15 +187,16 @@ def configure_warnings(enable):
         warnings.addHandler(logging.StreamHandler())
 
 
-def configure_test_logging():
-    """Add a NullHandler to root logger.
+def configure_test_logging(handler=None):
+    """Add a handler (defaults to NullHandler) to root logger.
 
     Prevents unconfigured logging from erroring, and swallows all logging,
     which is usually what you want for unit tests.  Unit test fixtures can
     still add their own loggers to assert against log messages if needed.
     """
     set_logger_class()
-    handler = logging.FileHandler('/dev/null')
+    if handler is None:
+        handler = logging.NullHandler()
     add_talisker_handler(logging.NOTSET, handler)
     configure_warnings(True)
 
@@ -198,48 +210,6 @@ def enable_debug_log_stderr():
     get_talisker_handler().setLevel(logging.DEBUG)
 
 
-# our enhanced version of the default raven support for recording breadcrumbs
-def record_log_breadcrumb(record):
-    # lazy import avoids any raven loggers being initialised early
-    from raven import breadcrumbs
-
-    breadcrumb_handler_args = (
-        logging.getLogger(record.name),
-        record.levelno,
-        record.message,
-        record.args,
-        {
-            'extra': getattr(record, '_structured', {}),
-            'exc_info': record.exc_info,
-            'stack_info': getattr(record, 'stack_info', None)
-        },
-    )
-
-    for handler in getattr(breadcrumbs, 'special_logging_handlers', []):
-        if handler(*breadcrumb_handler_args):
-            return
-
-    handler = breadcrumbs.special_logger_handlers.get(record.name)
-    if handler is not None and handler(*breadcrumb_handler_args):
-        return
-
-    def processor(data):
-        metadata = {
-            'path': record.pathname,
-            'lineno': record.lineno,
-        }
-        if hasattr(record, 'func'):
-            metadata['func'] = record.func
-        metadata.update(getattr(record, '_structured', {}))
-        data.update({
-            'message': record.message,
-            'category': record.name,
-            'level': record.levelname.lower(),
-            'data': metadata,
-        })
-    breadcrumbs.record(processor=processor)
-
-
 class StructuredLogger(logging.Logger):
     """A logger that handles passing 'extra' arguments to all logging calls.
 
@@ -253,6 +223,11 @@ class StructuredLogger(logging.Logger):
 
     """
 
+    def _log(self, *args, **kwargs):
+        t = time.time()
+        super()._log(*args, **kwargs)
+        track_request_metric('logging', get_rounded_ms(t))
+
     # sadly, we must subclass and override, rather that use the new
     # setLogRecordFactory() in 3.2+, as that does not pass the extra args
     # through. Also, we need to support python 2.
@@ -298,7 +273,8 @@ class StructuredLogger(logging.Logger):
         record = super(StructuredLogger, self).makeRecord(
             name, level, fn, lno, msg, args, exc_info, **kwargs)
         # store extra explicitly for StructuredFormatter to use
-        record._structured = structured
+        record.extra = structured
+        record._structured = structured  # b/w compat
         record._trailer = trailer
         return record
 
@@ -335,13 +311,15 @@ class StructuredFormatter(logging.Formatter):
 
     def format(self, record):
         """Format message with structured tags and any exception/trailer"""
+        import talisker.sentry  # lazy to break import cycle
         try:
             record.message = self.clean_message(record.getMessage())
 
             # we never want sentry to capture DEBUG logs in its breadcrumbs, as
             # they may be sensitive
             if record.levelno > logging.DEBUG:
-                record_log_breadcrumb(record)
+
+                talisker.sentry.record_log_breadcrumb(record)
 
             if len(record.message) > self.MAX_MSG_SIZE:
                 record.message = (
@@ -522,7 +500,7 @@ class StructuredFormatter(logging.Formatter):
         return s
 
 
-DEFAULT_COLORS = {
+DEFAULT_COLOURS = {
     'logfmt': '2;3;36',     # dim italic teal
     'name': '0;33',         # orange
     'msg': '1;16',          # bold white/black, depending on terminal palette
@@ -535,40 +513,40 @@ DEFAULT_COLORS = {
 }
 
 
-COLOR_SCHEMES = {}
-COLOR_SCHEMES['default'] = DEFAULT_COLORS
+COLOUR_SCHEMES = {}
+COLOUR_SCHEMES['default'] = DEFAULT_COLOURS
 
 # simple strips italics/bold
-COLOR_SCHEMES['simple'] = DEFAULT_COLORS.copy()
-COLOR_SCHEMES['simple']['logfmt'] = '0;36'
-COLOR_SCHEMES['simple']['msg'] = '0;37'
-COLOR_SCHEMES['simple']['time'] = '0;34'
+COLOUR_SCHEMES['simple'] = DEFAULT_COLOURS.copy()
+COLOUR_SCHEMES['simple']['logfmt'] = '0;36'
+COLOUR_SCHEMES['simple']['msg'] = '0;37'
+COLOUR_SCHEMES['simple']['time'] = '0;34'
 
 
-class ColoredFormatter(StructuredFormatter):
-    """Colorized log formatting"""
+class ColouredFormatter(StructuredFormatter):
+    """Colourised log formatting"""
     CLEAR = '\x1b[0m'
 
     def __init__(self, style='default'):
-        style = COLOR_SCHEMES[style]
-        self.colors = {k: '\x1b[' + v + 'm' for k, v in style.items()}
+        style = COLOUR_SCHEMES[style]
+        self.colours = {k: '\x1b[' + v + 'm' for k, v in style.items()}
         format = (
             '{time}%(asctime)s.%(msecs)03dZ{clear} '
-            '%(colored_levelname)s '
+            '%(coloured_levelname)s '
             '{name}%(name)s{clear} '
             '"{msg}%(message)s{clear}"'
-        ).format(clear=self.CLEAR, **self.colors)
+        ).format(clear=self.CLEAR, **self.colours)
         super().__init__(fmt=format)
 
     def format(self, record):
-        color = self.colors[record.levelname]
-        record.colored_levelname = '{color}{levelname}{clear}'.format(
-            color=color,
+        colour = self.colours[record.levelname]
+        record.coloured_levelname = '{colour}{levelname}{clear}'.format(
+            colour=colour,
             levelname=record.levelname,
             clear=self.CLEAR,
         )
-        return super(ColoredFormatter, self).format(record)
+        return super(ColouredFormatter, self).format(record)
 
     def logfmt(self, structured):
-        logfmt_str = super(ColoredFormatter, self).logfmt(structured)
-        return self.colors['logfmt'] + logfmt_str + self.CLEAR
+        logfmt_str = super(ColouredFormatter, self).logfmt(structured)
+        return self.colours['logfmt'] + logfmt_str + self.CLEAR
